==> reywechat-0.51-py3-none-any.whl: reywechat/rclient.py <==
# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@Time    : 2023-10-17 20:27:16
@Author  : Rey
@Contact : reyxbo@163.com
@Explain : Client methods.
"""


from __future__ import annotations
from typing import Any, List, Tuple, Dict, Optional, Literal, Union
from os.path import abspath as os_abspath
from reytool.rcomm import request as reytool_request
from reytool.rdll import inject_dll
from reytool.rsystem import throw, search_process
from reytool.ros import find_relpath

from .rwechat import RWeChat


__all__ = (
    "RClient",
)


class RClientErorr(Exception):
    """
    Rey's `client` exception type.
    """


class RClient(object):
    """
    Rey's `client` type.
    """


    def __init__(
        self,
        rwechat: RWeChat
    ) -> None:
        """
        Build `client` instance.

        Parameters
        ----------
        rwechat : `RWeChat` instance.
        """

        # Start.
        self.rwechat = rwechat
        self.start_api()

        # Set attribute.
        self.login_info = self.get_login_info()


    def start_api(self) -> None:
        """
        Start client control API.
        """

        # Check client.
        result = self.check_client()
        if not result:
            raise RClientErorr("WeChat client not started")

        # Check start.
        result = self.check_api()
        if not result:

            # Inject DLL.
            self.inject_dll()

            # Check api.
            result = self.check_api()
            if not result:
                raise RClientErorr("start WeChat client API failed")

        # Report.
        print("Start WeChat client API successfully, address is '127.0.0.1:19088'.")


    def check_client(self) -> bool:
        """
        Check if the client is started.

        Returns
        -------
        Check result.
        """

        # Search.
        processes = search_process(name="WeChat.exe")

        # Check.
        if processes == []:
            return False
        else:
            return True


    def check_api(self) -> bool:
        """
        Check if the client API is started.
        """

        # Search.
        processes = search_process(port=self.rwechat.client_api_port)

        # Check.
        if processes == []:
            return False
        process = processes[0]
        with process.oneshot():
            process_name = process.name()
        if process_name != "WeChat.exe":
            return False

        ## Check request.
        result = self.check_login()
        if not result:
            return False

        return True


    def inject_dll(self) -> None:
        """
        Inject DLL file of start API into the WeChat client process.
        """

        # Get parameter.
        dll_file_relpath = ".\\data\\client_api.dll"
        dll_file_path = find_relpath(__file__, dll_file_relpath)

        # Inject.
        processes = search_process(name="WeChat.exe")
        process = processes[0]
        inject_dll(
            process.pid,
            dll_file_path
        )


    def request(
        self,
        api: str,
        data: Optional[Dict] = None,
        success_code: Optional[Union[int, List[int]]] = None
    ) -> Dict[
        Literal["code", "message", "data"],
        Any
    ]:
        """
        Request client API.

        Parameters
        ----------
        api : API name.
        data : Request data.
        success_code : Suceess code, if unsuccessful, throw an exception.
            - `None` : Not handle.
            - `Union[int, List[int]]` : Handle.

        Returns
        -------
        Response content.
            - `code` : Response code.
            - `message` : Response message.
            - `data` : Response data.
        """

        # Get parameter.
        url = f"http://127.0.0.1:{self.rwechat.client_api_port}/api/{api}"
        if data is None:
            data = {}
        if success_code.__class__ == int:
            success_code = [success_code]

        # Request.
        response = reytool_request(
            url,
            json=data,
            method="post",
            check=True
        )

        # Extract.
        response_data = response.json()
        response = {
            "code": response_data["code"],
            "message": response_data["msg"],
            "data": response_data["data"]
        }

        # Raise.
        if (
            success_code is not None
            and response["code"] not in success_code
        ):
            raise RClientErorr(f"client API '{api}' request failed", data, response)

        return response


    def check_login(self) -> bool:
        """
        Check if the client is logged in.

        Returns
        -------
        Check result.
        """

        # Get parameter.
        api = "checkLogin"

        # Request.
        response = self.request(api)

        # Check.
        if response["code"] == 1:
            return True
        elif response["code"] == 0:
            return False


    def get_login_info(
        self
    ) -> Dict[
        Literal[
            "user_id",
            "account",
            "name",
            "phone",
            "signature",
            "city",
            "province",
            "country",
            "head_image",
            "account_data_path",
            "wechat_data_path",
            "decrypt_key"
        ],
        Optional[str]
    ]:
        """
        Get login account information.

        Returns
        -------
        Login user account information.
            - `Key 'user_id'` : User ID, cannot change.
            - `Key 'account' : User account, can change.
            - `Key 'name' : User nickname.
            - `Key 'phone' : Phone number.
            - `Key 'signature' : Personal signature.
            - `Key 'city' : City.
            - `Key 'province' : Province.
            - `Key 'country' : Country.
            - `Key 'head_image' : Head image URL.
            - `Key 'account_data_path' : Current account data save path.
            - `Key 'wechat_data_path' : WeChat data save path.
            - `Key 'decrypt_key' : Database decrypt key.
        """

        # Get parameter.
        api = "userInfo"

        # Request.
        response = self.request(api)

        # Extract.
        data = response["data"]
        info = {
            "user_id": data["wxid"],
            "account": data["account"],
            "name": data["name"],
            "phone": data["mobile"],
            "signature": data["signature"],
            "city": data["city"],
            "province": data["province"],
            "country": data["country"],
            "head_image": data["headImage"],
            "account_data_path": data["currentDataPath"],
            "wechat_data_path": data["dataSavePath"],
            "decrypt_key": data["dbKey"]
        }
        info = {
            key: (
                None
                if value == ""
                else value
            )
            for key, value in info.items()
        }

        return info


    def hook_message(
        self,
        host: str,
        port: Union[str, int],
        timeout: float
    ) -> None:
        """
        Hook the message, and send the message to the TCP protocol request.

        Parameters
        ----------
        host : Request host.
        port : Request port.
        timeout : Request timeout seconds.
        """

        # Get parameter.
        api = "hookSyncMsg"
        port = str(port)
        timeout_ms_str = str(int(timeout * 1000))

        # Request.
        data = {
            "ip": host,
            "port": port,
            "timeout": timeout_ms_str,
            "enableHttp": "0"
        }
        response = self.request(api, data, [0, 2])

        # Check.
        if response["code"] == 2:
            self.unhook_message()
            self.hook_message(
                host,
                port,
                timeout
            )

        # Report.
        print(
            "Hook message successfully, address is '%s:%s'." % (
                host,
                port
            )
        )


    def unhook_message(self) -> None:
        """
        Unhook the message.
        """

        # Get parameter.
        api = "unhookSyncMsg"

        # Request.
        self.request(api, success_code=0)

        # Report.
        print("Unhook message successfully.")


    def download_file(
        self,
        id_: int
    ) -> None:
        """
        Download image or video or other file.

        Parameters
        ----------
        id_ : Message ID.
        """

        # Get parameter.
        api = "downloadAttach"

        # Request.
        data = {"msgId": id_}
        self.request(api, data, 0)


    def download_voice(
        self,
        id_: int,
        dir_: str
    ) -> None:
        """
        Download voice.

        Parameters
        ----------
        id_ : Message ID.
        dir_ : Save directory.
        """

        # Get parameter.
        api = "getVoiceByMsgId"
        dir_ = os_abspath(dir_)

        # Request.
        data = {
            "msgId": id_,
            "storeDir": dir_
        }
        self.request(api, data, [0, 1])


    def get_contact_table(
        self,
        type_: Optional[Literal["user", "room"]] = None
    ) -> List[
        Dict[
            Literal["id", "name"],
            str
        ]
    ]:
        """
        Get contact table, include chat user and chat room.

        Parameters
        ----------
        type_ : Return contact table type.
            - `None` : Return all contact table.
            - `Literal['user']` : Return user contact table.
            - `Literal['room']` : Return chat room contact table.

        Returns
        -------
        Contact table.
            - `Key 'id'` : User ID or chat room ID.
            - `Key 'name' : User nickname or chat room name.
        """

        # Get parameter.
        api = "getContactList"

        # Request.
        response = self.request(api, success_code=1)

        # Extract.
        data: List[Dict] = response["data"]
        table = [
            {
                "id": info["wxid"],
                "name": info["nickname"]
            }
            for info in data
        ]

        ## User.
        if type_ == "user":
            table = [
                row
                for row in table
                if row["id"][-9:] != "@chatroom"
            ]

        ## Chat room.
        elif type_ == "room":
            table = [
                row
                for row in table
                if row["id"][-9:] == "@chatroom"
            ]

        ## Raise.
        elif type_ is not None:
            throw(ValueError, type_)

        # Filter system user.
        filter_names = (
            "朋友推荐消息",
            "语音记事本",
            "漂流瓶",
            "文件传输助手"
        )
        for name in filter_names:
            for row in table:
                if row["name"] == name:
                    table.remove(row)
                    break

        # Chat room default name.
        if type_ in (None, "room"):
            for row in table:
                if (
                    row["name"] == ""
                    and row["id"][-9:] == "@chatroom"
                ):
                    row["name"] = "群聊"

        return table


    def get_contact_name(
        self,
        id_: str
    ) -> str:
        """
        Get contact name, can be friend and chat room and chat room member.

        Parameters
        ----------
        id_ : User ID or chat room ID.

        Returns
        -------
        User nickname or chat room name.
        """

        # Get parameter.
        api = "getContactProfile"

        # Request.
        data = {"wxid": id_}
        response = self.request(api, data, 1)

        # Extract.
        data: dict = response["data"]
        name = data["nickname"]

        # Chat room default name.
        if (
            name == ""
            and id_[-9:] == "@chatroom"
        ):
            name = "群聊"

        return name


    def get_room_member_list(
        self,
        room_id: str
    ) -> List[str]:
        """
        Get list of chat room member user ID.

        Parameters
        ----------
        room_id : Chat room ID.

        Returns
        -------
        List of chat room member user ID.
        """

        # Get parameter.
        api = "getMemberFromChatRoom"

        # Request.
        data = {"chatRoomId": room_id}
        response = self.request(api, data, 1)

        # Convert.
        data: Dict = response["data"]
        members_list = data["members"].split("^G")

        return members_list


    def get_room_member_table(
        self,
        room_id: str
    ) -> Dict[str, str]:
        """
        Get table of chat room member user ID and user name.

        Parameters
        ----------
        room_id : Chat room ID.

        Returns
        -------
        Table of chat room member user ID and user name.
        """

        # Get member.
        members = self.get_room_member_list(room_id)

        # Loop.
        table = {
            id_: self.get_contact_name(id_)
            for id_ in members
            if id_ != ""
        }

        return table

==> reywechat-0.54-py3-none-any.whl: reywechat/rclient.py <==
# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@Time    : 2023-10-17 20:27:16
@Author  : Rey
@Contact : reyxbo@163.com
@Explain : Client methods.
"""


from __future__ import annotations
from typing import Any, List, Dict, Optional, Literal, Union, overload
from os.path import abspath as os_abspath
from reytool.rcomm import request as reytool_request
from reytool.rdll import inject_dll
from reytool.rsystem import throw, search_process
from reytool.ros import find_relpath

from .rwechat import RWeChat


__all__ = (
    "RClient",
)


# Response content dictionary, include response code and response message and response data.
Response = Dict[
    Literal["code", "message", "data"],
    Any
]


class RClientErorr(Exception):
    """
    Rey's `client` exception type.
    """


class RClient(object):
    """
    Rey's `client` type.
    """


    def __init__(
        self,
        rwechat: RWeChat
    ) -> None:
        """
        Build `client` instance.

        Parameters
        ----------
        rwechat : `RWeChat` instance.
        """

        # Start.
        self.rwechat = rwechat
        self.start_api()

        # Set attribute.
        self.login_info = self.get_login_info()


    def start_api(self) -> None:
        """
        Start client control API.
        """

        # Check client.
        result = self.check_client()
        if not result:
            raise RClientErorr("WeChat client not started")

        # Check start.
        result = self.check_api()
        if not result:

            # Inject DLL.
            self.inject_dll()

            # Check api.
            result = self.check_api()
            if not result:
                raise RClientErorr("start WeChat client API failed")

        # Report.
        print("Start WeChat client API successfully, address is '127.0.0.1:19088'.")


    def check_client(self) -> bool:
        """
        Check if the client is started.

        Returns
        -------
        Check result.
        """

        # Search.
        processes = search_process(name="WeChat.exe")

        # Check.
        if processes == []:
            return False
        else:
            return True


    def check_api(self) -> bool:
        """
        Check if the client API is started.
        """

        # Search.
        processes = search_process(port=self.rwechat.client_api_port)

        # Check.
        if processes == []:
            return False
        process = processes[0]
        with process.oneshot():
            process_name = process.name()
        if process_name != "WeChat.exe":
            return False

        ## Check request.
        result = self.check_login()
        if not result:
            return False

        return True


    def inject_dll(self) -> None:
        """
        Inject DLL file of start API into the WeChat client process.
        """

        # Get parameter.
        dll_file_relpath = ".\\data\\client_api.dll"
        dll_file_path = find_relpath(__file__, dll_file_relpath)

        # Inject.
        processes = search_process(name="WeChat.exe")
        process = processes[0]
        inject_dll(
            process.pid,
            dll_file_path
        )


    def request(
        self,
        api: str,
        data: Optional[Dict] = None,
        success_code: Optional[Union[int, List[int]]] = None
    ) -> Response:
        """
        Request client API.

        Parameters
        ----------
        api : API name.
        data : Request data.
        success_code : Suceess code, if unsuccessful, throw an exception.
            - `None` : Not handle.
            - `Union[int, List[int]]` : Handle.

        Returns
        -------
        Response content dictionary.
        """

        # Get parameter.
        url = f"http://127.0.0.1:{self.rwechat.client_api_port}/api/{api}"
        if data is None:
            data = {}
        if success_code.__class__ == int:
            success_code = [success_code]

        # Request.
        response = reytool_request(
            url,
            json=data,
            method="post",
            check=True
        )

        # Extract.
        response_data = response.json()
        response = {
            "code": response_data["code"],
            "message": response_data["msg"],
            "data": response_data["data"]
        }

        # Raise.
        if (
            success_code is not None
            and response["code"] not in success_code
        ):
            raise RClientErorr(f"client API '{api}' request failed", data, response)

        return response


    def check_login(self) -> bool:
        """
        Check if the client is logged in.

        Returns
        -------
        Check result.
        """

        # Get parameter.
        api = "checkLogin"

        # Request.
        response = self.request(api)

        # Check.
        if response["code"] == 1:
            return True
        elif response["code"] == 0:
            return False


    def get_login_info(
        self
    ) -> Dict[
        Literal[
            "user_id",
            "account",
            "name",
            "phone",
            "signature",
            "city",
            "province",
            "country",
            "head_image",
            "account_data_path",
            "wechat_data_path",
            "decrypt_key"
        ],
        Optional[str]
    ]:
        """
        Get login account information.

        Returns
        -------
        Login user account information.
            - `Key 'user_id'` : User ID, cannot change.
            - `Key 'account' : User account, can change.
            - `Key 'name' : User nickname.
            - `Key 'phone' : Phone number.
            - `Key 'signature' : Personal signature.
            - `Key 'city' : City.
            - `Key 'province' : Province.
            - `Key 'country' : Country.
            - `Key 'head_image' : Head image URL.
            - `Key 'account_data_path' : Current account data save path.
            - `Key 'wechat_data_path' : WeChat data save path.
            - `Key 'decrypt_key' : Database decrypt key.
        """

        # Get parameter.
        api = "userInfo"

        # Request.
        response = self.request(api)

        # Extract.
        data = response["data"]
        info = {
            "user_id": data["wxid"],
            "account": data["account"],
            "name": data["name"],
            "phone": data["mobile"],
            "signature": data["signature"],
            "city": data["city"],
            "province": data["province"],
            "country": data["country"],
            "head_image": data["headImage"],
            "account_data_path": data["currentDataPath"],
            "wechat_data_path": data["dataSavePath"],
            "decrypt_key": data["dbKey"]
        }
        info = {
            key: (
                None
                if value == ""
                else value
            )
            for key, value in info.items()
        }

        return info


    def hook_message(
        self,
        host: str,
        port: Union[str, int],
        timeout: float
    ) -> None:
        """
        Hook the message, and send the message to the TCP protocol request.

        Parameters
        ----------
        host : Request host.
        port : Request port.
        timeout : Request timeout seconds.
        """

        # Get parameter.
        api = "hookSyncMsg"
        port = str(port)
        timeout_ms_str = str(int(timeout * 1000))

        # Request.
        data = {
            "ip": host,
            "port": port,
            "timeout": timeout_ms_str,
            "enableHttp": "0"
        }
        response = self.request(api, data, [0, 2])

        # Check.
        if response["code"] == 2:
            self.unhook_message()
            self.hook_message(
                host,
                port,
                timeout
            )

        # Report.
        print(
            "Hook message successfully, address is '%s:%s'." % (
                host,
                port
            )
        )


    def unhook_message(self) -> None:
        """
        Unhook the message.
        """

        # Get parameter.
        api = "unhookSyncMsg"

        # Request.
        self.request(api, success_code=0)

        # Report.
        print("Unhook message successfully.")


    def download_file(
        self,
        id_: int
    ) -> None:
        """
        Download image or video or other file.

        Parameters
        ----------
        id_ : Message ID.
        """

        # Get parameter.
        api = "downloadAttach"

        # Request.
        data = {"msgId": id_}
        self.request(api, data, 0)


    def download_voice(
        self,
        id_: int,
        dir_: str
    ) -> None:
        """
        Download voice.

        Parameters
        ----------
        id_ : Message ID.
        dir_ : Save directory.
        """

        # Get parameter.
        api = "getVoiceByMsgId"
        dir_ = os_abspath(dir_)

        # Request.
        data = {
            "msgId": id_,
            "storeDir": dir_
        }
        self.request(api, data, [0, 1])


    def get_contact_table(
        self,
        type_: Optional[Literal["user", "room"]] = None
    ) -> List[
        Dict[
            Literal["id", "name"],
            str
        ]
    ]:
        """
        Get contact table, include chat user and chat room.

        Parameters
        ----------
        type_ : Return contact table type.
            - `None` : Return all contact table.
            - `Literal['user']` : Return user contact table.
            - `Literal['room']` : Return chat room contact table.

        Returns
        -------
        Contact table.
            - `Key 'id'` : User ID or chat room ID.
            - `Key 'name' : User nickname or chat room name.
        """

        # Get parameter.
        api = "getContactList"
        filter_names = {
            "filehelper": "朋友推荐消息",
            "floatbottle": "语音记事本",
            "fmessage": "漂流瓶",
            "medianote": "文件传输助手"
        }

        # Request.
        response = self.request(api, success_code=1)

        # Extract.
        data: List[Dict] = response["data"]
        table_user = []
        table_room = []
        for info in data:
            id_ = info["wxid"]

            # Filter system user.
            if id_ in filter_names:
                continue

            # Split table.
            row = {
                "id": id_,
                "name": info["nickname"]
            }

            ## Chat room table.
            if id_[-9:] == "@chatroom":
                if type_ in (None, "room"):
                    table_room.append(row)

            ## User table.
            else:
                if type_ in (None, "user"):
                    table_user.append(row)

        # User no name.
        for row in table_user[::-1]:
            if row["name"] == "":
                table_user.remove(row)

        # Chat room default name.
        for row in table_room:
            if row["name"] == "":
                row["name"] = "群聊"

        # Merge table.
        table = table_user + table_room

        return table


    @overload
    def get_contact_name(
        self,
        id_: str,
        full: Literal[False] = False
    ) -> str: ...

    @overload
    def get_contact_name(
        self,
        id_: str,
        full: Literal[True] = False
    ) -> Response: ...

    def get_contact_name(
        self,
        id_: str,
        full: bool = False
    ) -> Union[str, Response]:
        """
        Get contact name, can be friend and chat room and chat room member.

        Parameters
        ----------
        id_ : User ID or chat room ID.
        full : Whether return full response content.

        Returns
        -------
        User nickname or chat room name.
        """

        # Get parameter.
        api = "getContactProfile"

        # Request.
        data = {"wxid": id_}

        ## Not check.
        if full:
            response = self.request(api, data)
            return response

        ## Check.
        else:
            response = self.request(api, data, 1)

        # Extract.
        data: dict = response["data"]
        name = data["nickname"]

        # Chat room default name.
        if (
            name == ""
            and id_[-9:] == "@chatroom"
        ):
            name = "群聊"

        return name


    def get_room_member_list(
        self,
        room_id: str
    ) -> List[str]:
        """
        Get list of chat room member user ID.

        Parameters
        ----------
        room_id : Chat room ID.

        Returns
        -------
        List of chat room member user ID.
        """

        # Get parameter.
        api = "getMemberFromChatRoom"

        # Request.
        data = {"chatRoomId": room_id}
        response = self.request(api, data, 1)

        # Convert.
        data: Dict = response["data"]
        members: str = data["members"]
        members_list = members.split("^G")
        members_list = list(filter(
            lambda member: member != "",
            members_list
        ))

        return members_list


    def get_room_member_table(
        self,
        room_id: str
    ) -> Dict[str, str]:
        """
        Get table of chat room member user ID and user name.

        Parameters
        ----------
        room_id : Chat room ID.

        Returns
        -------
        Table of chat room member user ID and user name.
        """

        # Get member.
        members = self.get_room_member_list(room_id)

        # Loop.
        table = {}
        for member in members:
            response = self.get_contact_name(member, True)
            if response["code"] == 0:
                name = None
            else:
                name = response["data"]["nickname"]
            table[member] = name

        return table
